package org.qatools.factory;

import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;
import org.qatools.util.ListUtil;

public class AllCombinations extends QAToolsFactory {
    private static Logger logger = Logger.getLogger(AllCombinations.class);


    // --------------------------------------------------------
    // ----- print
    // --------------------------------------------------------

    /**
     * 
     * @param allCombinations
     */
    public static void print(int[][] indexes) {
    	int rows = indexes.length;
    	int cols = indexes[0].length;
    	System.out.println("-----------------------------");
    	System.out.println("rows: " + rows);
    	System.out.println("cols: " + cols);
    	for (int i=0; i<indexes.length; i++) {
    		int[] row = indexes[i]; 		
    		String s = null;
    		s = (cols==1 ? "" + row[0] : s);
    		s = (cols==2 ? "" + row[0] + "," + row[1] : s);
    		s = (cols==3 ? "" + row[0] + "," + row[1] + "," + row[2] : s);
    		s = (cols==4 ? "" + row[0] + "," + row[1] + "," + row[2] + "," + row[3] : s);
    		
    		System.out.println(s);
    	}
    }

    /**
     * 
     * @param allCombinations
     */
    public static void print(Object[][] combinations) {
    	int rows = combinations.length;
    	int cols = combinations[0].length;
    	System.out.println("-----------------------------");
    	System.out.println("rows: " + rows);
    	System.out.println("cols: " + cols);
    	for (int i=0; i<combinations.length; i++) {
    		Object[] row = combinations[i]; 		
    		String s = ListUtil.getCommaDelimitedStr(Arrays.asList(row));
    		System.out.println(s);
    	}
    }


    // --------------------------------------------------------
    // ----- two params
    // --------------------------------------------------------

    /**
     * 
     * @param objs
     * @param objs2
     * @return
     */
    public static <A extends Object, B extends Object> Object[][] allCombinations(A[] objs, B[] objs2) {
    	return allCombinations(Arrays.asList(objs), Arrays.asList(objs2));
    }
    
    /**
     * 
     * @param list
     * @param list2
     * @return
     */
    public static <A extends Object, B extends Object> Object[][] allCombinations(List<A> list, List<B> list2) {
    	return allCombinations(list, list2, null);
    }
    
    // --------------------------------------------------------
    // ----- three params
    // --------------------------------------------------------

    /**
     * 
     * @param objs
     * @param objs2
     * @return
     */
    public static <A extends Object, B extends Object, C extends Object> Object[][] allCombinations(A[] objs, B[] objs2, C[] objs3) {
    	return allCombinations(Arrays.asList(objs), Arrays.asList(objs2), Arrays.asList(objs3));
    }
    
    /**
     * 
     * @param list
     * @param list2
     * @return
     */
    public static <A extends Object, B extends Object, C extends Object> Object[][] allCombinations(List<A> list, List<B> list2, List<C> list3) {
    	return allCombinations(list, list2, list3, null);
    }
	
    // --------------------------------------------------------
    // ----- four params
    // --------------------------------------------------------

    /**
     * 
     * @param objs
     * @param objs2
     * @return
     */
    public static <A extends Object, B extends Object, C extends Object, D extends Object> Object[][] allCombinations(A[] objs, B[] objs2, C[] objs3, D[] objs4) {
    	return allCombinations(Arrays.asList(objs), Arrays.asList(objs2), Arrays.asList(objs3), Arrays.asList(objs4));
    }
    
//    /**
//     * 
//     * @param list
//     * @param list2
//     * @return
//     */
//    public static <A extends Object, B extends Object, C extends Object, D extends Object> Object[][] allCombinations2(List<A> list, List<B> list2, List<C> list3, List<D> list4) {
//    	if (list==null || list.size()==0 || list2==null || list2.size()==0) {
//    		throw new RuntimeException("Need at least 2 parameters to create allCombinations");
//    	}
//    	
//    	//
//    	// determine number of rows & columns in the response
//    	int numCols = 2;
//    	int numRows = list.size() * list2.size();
//    	if (list3!=null && 0<list3.size()) {
//    		numCols++;
//    		numRows *= list3.size();
//    		
//    		if (list4!=null && 0<list4.size()) {
//    			numCols++;
//    			numRows *= list4.size();
//    		}
//    	}
//    	
//    	//
//    	//
//    	Object[][] result = new Object[numRows][numCols];
//    	int i=0;
//    	for (int a=0; a<list.size(); a++) {
//    		Object aa = list.get(a);
//    		
//    		for (int b=0; b<list2.size(); b++) {
//    			Object bb = list2.get(b);
//    			
//    			if (3 <= numCols) {
//    				for (int c=0; c<list3.size(); c++) {
//    					Object cc = list3.get(c);
//    					
//    					if (4 <= numCols) {
//    						for (int d=0; d<list4.size(); d++) {
//    							Object dd = list4.get(d);
//    							
//        						// four params
//        	    				result[i][0] = aa;
//        	    				result[i][1] = bb;
//        	    				result[i][2] = cc;
//        	    				result[i][3] = dd;
//        	    				i++;
//    						}	//d
//    					} else {
//    						// three params
//    	    				result[i][0] = aa;
//    	    				result[i][1] = bb;
//    	    				result[i][2] = cc;
//    	    				i++;
//    					} //4
//    				}	//c
//    			} else {
//    				// two params
//    				result[i][0] = aa;
//    				result[i][1] = bb;
//    				i++;
//    			} //3
//    		}	//b
//    	}	//a
//    	
//    	return result;
//    }

    /**
     * 
     * @param list
     * @param list2
     * @return
     */
    protected static <A extends Object, B extends Object, C extends Object, D extends Object> Object[][] allCombinations(List<A> list, List<B> list2, List<C> list3, List<D> list4) {
    	int[][] indexes = allCombinationsIndexes(list, list2, list3, list4);
    	int rows = indexes.length;
    	int cols = indexes[0].length;
    	
    	Object[][] result = new Object[rows][cols];
    	for (int a=0; a<rows; a++) {
    		for (int b=0; b<cols; b++) {
    			Object o = null;
    			o = (b==0 ? list.get(indexes[a][b]) : o);
    			o = (b==1 ? list2.get(indexes[a][b]) : o);
    			o = (b==2 ? list3.get(indexes[a][b]) : o);
    			o = (b==3 ? list4.get(indexes[a][b]) : o);
    			result[a][b] = o;
    		}
    	}
    	return result;
    }

    
    /**
     * 
     * @param list
     * @param list2
     * @return
     */
    protected static <A extends Object, B extends Object, C extends Object, D extends Object> int[][] allCombinationsIndexes(List<A> list, List<B> list2, List<C> list3, List<D> list4) {
    	if (list==null || list.size()==0 || list2==null || list2.size()==0) {
    		throw new RuntimeException("Need at least 2 parameters to create allCombinations");
    	}
    	
    	//
    	// determine number of rows & columns in the response
    	int numCols = 2;
    	int numRows = list.size() * list2.size();
    	if (list3!=null && 0<list3.size()) {
    		numCols++;
    		numRows *= list3.size();
    		
    		if (list4!=null && 0<list4.size()) {
    			numCols++;
    			numRows *= list4.size();
    		}
    	}
    	
    	//
    	//
    	int[][] result = new int[numRows][numCols];
    	int i=0;
    	for (int a=0; a<list.size(); a++) {
    		//Object aa = list.get(a);
    		
    		for (int b=0; b<list2.size(); b++) {
    			//Object bb = list2.get(b);
    			
    			if (3 <= numCols) {
    				for (int c=0; c<list3.size(); c++) {
    					//Object cc = list3.get(c);
    					
    					if (4 <= numCols) {
    						for (int d=0; d<list4.size(); d++) {
    							//Object dd = list4.get(d);
    							
        						// four params
        	    				result[i][0] = a;
        	    				result[i][1] = b;
        	    				result[i][2] = c;
        	    				result[i][3] = d;
        	    				i++;
    						}	//d
    					} else {
    						// three params
    	    				result[i][0] = a;
    	    				result[i][1] = b;
    	    				result[i][2] = c;
    	    				i++;
    					} //4
    				}	//c
    			} else {
    				// two params
    				result[i][0] = a;
    				result[i][1] = b;
    				i++;
    			} //3
    		}	//b
    	}	//a
    	
    	return result;
    }

    
    
}